`timescale 10ps / 10ps
`define GTK // use VCD

`include "rtavr_defs.v"

`define MODULE	tb_007

module `MODULE;

   reg CLK2X;  // input
   reg RESET;  // input
   wire [11:0] ROM_ADDRA; // output
   reg [15:0] ROM_DOA; // input
   wire ROM_WEB; // output
   wire ROM_OE; // output
   wire [12:0] ROM_ADDRB; // output
   wire [7:0] ROM_DIB; // output
   wire [7:0] ROM_DOB = 0; // input
   wire [4:0] INT_VEC; // output
   wire INT_REQ; // output
   wire INT_ACK = 0; // input

rtavr top (.RESET(RESET), .CLK2X(CLK2X)
		, .ROM_ADDRA(ROM_ADDRA[11:0]) 
		, .ROM_DOA(ROM_DOA[15:0])
		, .ROM_WEB(ROM_WEB), .ROM_OE(ROM_OE)
		, .ROM_ADDRB(ROM_ADDRB[12:0]) 
		, .ROM_DIB(ROM_DIB[7:0])
		, .ROM_DOB(ROM_DOB[7:0])
		, .INT_VEC(INT_VEC[4:0])
		, .INT_REQ(INT_REQ)
		, .INT_ACK(INT_ACK)
 );

   parameter CLKP = 5000;        //  50ns
   parameter CLKH = CLKP/2;      //
   parameter CLKL = CLKP-CLKH;   //
   
   always begin
      #CLKH CLK2X = 0;
      #CLKL CLK2X = 1;
   end

`ifdef GTK
   initial begin
      $dumpfile("tb.vcd");
      $dumpvars(0,`MODULE.top);
   end
`endif

   initial begin : test
      // LDI             1110:KKKK:dddd:KKKK
      sys_reset(16'he51a);  // LDI r17, 0x5a
      // SWAP            1001:010d:dddd:0010     #
      // DEC             1001:010d:dddd:1010     #
      //rom_out(16'h9512);  // SWAP r17
      rom_out(16'h951a);  // DEC, r17
      // MOV             0010:11rd:dddd:rrrr     #
      rom_out(16'h2f21);  // MOV r18,r17
      // SUBI            0101:KKKK:dddd:KKKK     #
      rom_out(16'h5510);  // SUBI r17, 0x50
      // ADD/ADC         000C:11rd:dddd:rrrr     #
      rom_out(16'h0f21);  // ADD r18,r17

      // LDI             1110:KKKK:dddd:KKKK
      rom_out(16'he0a0);  // LDI r26, 0x00
      rom_out(16'he0b1);  // LDI r27, 0x01
      // ST.X++.Rr       1001:001r:rrrr:1101     #
      rom_out(16'h931d);  // ST X++, r17         # X 0x0100 -> 0x0101
      rom_out(16'h931d);  // reentrant

      // ST.X.Rr         1001:001r:rrrr:1100     #
      rom_out(16'h932c);  // ST X, r18           # X 0x0101
      // ST.--X.Rr       1001:001r:rrrr:1110     #
      rom_out(16'h931e);  // ST --X, r17         # X 0x0100
      // ST.--X.Rr       1001:001r:rrrr:1110     #
      rom_out(16'h932e);  // ST --X, r18         # X 0x00ff 

      rom_out(16'h913c);  // LD X, r19           # X 0x00ff 
//      rom_out(16'h913c);  // reentrant (-> not conflict)
      // LD.Rd.--X       1001:000d:dddd:1110
      rom_out(16'h914e);  // LD --X, r20         # X 0x00fe
      rom_out(16'h914e);  // reentrant
      // LD.Rd.X++       1001:000d:dddd:1101     #
      rom_out(16'h915d);  // LD X++, r21         # X 0x00fe -> 0x00ff
      rom_out(16'h915d);  // reentrant
      rom_out(16'h916d);  // LD X++, r22         # X 0x00ff -> 0x0100
      rom_out(16'h916d);  // reentrant
      // LD.Rd.X         1001:000d:dddd:1100     #
      rom_out(16'h917c);  // LD X, r23           # X 0x0100 
      rom_out(16'h917c);  // reentrant -> 2 times if defined NO_STALL_IDX

      // RJMP            1100:kkkk:kkkk:kkkk     #
      rom_out(16'hc100);  // RJMP +100
      rom_out(16'h0000);  // NOP

      // LDI             1110:KKKK:dddd:KKKK
      rom_out(16'he2e0);  // LDI r30, 0x20
      rom_out(16'he0f2);  // LDI r31, 0x02

      // IJMP            1001:0100:0000:1001     #
      rom_out(16'h9409);  // IJMP
      rom_out(16'h0000);  // NOP
      // SBRC            1111:110r:rrrr:0bbb     #
      // SBRS            1111:111r:rrrr:0bbb     #
      rom_out(16'hfdf0);  // SBRC r31,0
      rom_out(16'hfdf1);  // SBRC r31,1
      rom_out(16'h0000);  // NOP
      rom_out(16'hfff1);  // SBRS r31,1
      rom_out(16'hfff0);  // SBRS r31,0
      rom_out(16'h0000);  // NOP
// tb_006
      // IN              1011:0AA1:dddd:AAAA     # 00
      // OUT             1011:1AA1:rrrr:AAAA     # 00
      // LDS             1010:0kkK:dddd:kkkk     # 
      // STS             1010:1kkK:dddd:kkkk     # 
      // LDI             1110:KKKK:dddd:KKKK
      rom_out(16'hebcc);  // LDI r28, 0xbc
      rom_out(16'he0d0);  // LDI r29, 0x00
      rom_out(16'hbfcd);  // OUT SPL , r28
      rom_out(16'hbfde);  // OUT SPH , r29
      rom_out(16'hafcc);  // STS 0xbc, r28
      rom_out(16'hafdd);  // STS 0xbd, r29
      // LD.Rd.Y         1000:000d:dddd:1000     #
      // LD.Rd.--Y       1001:000d:dddd:1010
      rom_out(16'h8138);  // LD r19,Y
      rom_out(16'h0000);  // NOP
      // MOV             0010:11rd:dddd:rrrr     #
      rom_out(16'h2f43);  // MOV r20,r19
      rom_out(16'h8138);  // LD r19,Y


      // BST             1111:101d:dddd:0bbb     #
      rom_out(16'hfb42);  // BST 2, r20
      // BLD             1111:100d:dddd:0bbb     #
      rom_out(16'hf930);  // BLD r19, 0
      // BSET            1001:0100:0sss:1000     #
      rom_out(16'h9478);  // BSET I (STI)
      // BCLR            1001:0100:1sss:1000     #
      rom_out(16'h94f8);  // BCLR I (CLI)

      // BRBC            1111:01kk:kkkk:ksss     #
      rom_out(16'hf487);  // BRBC I
      rom_out(16'h0000);  // NOP
      // BRBS            1111:00kk:kkkk:ksss     #
      rom_out(16'hf087);  // BRBS I
      rom_out(16'h0000);  // NOP
// tb_007
      // PUSH            1001:001d:dddd:1111     #
      rom_out(16'h933f);  // PUSH r19
      rom_out(16'h934f);  // PUSH r20
      // POP             1001:000d:dddd:1111     #
      rom_out(16'h913f);  // POP r19
      rom_out(16'h914f);  // POP r20

      // ICALL           1001:0101:0000:1001     #
      rom_out(16'h9509);  // ICALL
      rom_out(16'h0000);  // NOP
      rom_out(16'h0000);  // NOP
      // RET             1001:0101:0000:1000    #
      // RETI            1001:0101:0001:1000    #
      rom_out(16'h9508);  // RET
      rom_out(16'h0000);  // NOP
      rom_out(16'h0000);  // NOP

      rom_out(16'h0000);  // NOP
      rom_out(16'h0000);  // NOP
      rom_out(16'h0000);  // NOP
      $display("*** Simulation Complete! ***");
      $finish;
   end

   task sys_reset;
      input [15:0] data;
      begin
         #1 RESET = 0;
         CLK2X = 0;
         @(posedge CLK2X);
         @(posedge CLK2X);
	 @(negedge CLK2X);
         #1 ROM_DOA[15:0] = data[15:0];
         @(posedge CLK2X);
         #1 RESET = 1;
         @(posedge CLK2X);
         @(posedge CLK2X);
      end
   endtask // sys_reset
   
   task rom_out;
      input [15:0] data;
      begin
	 @(posedge CLK2X);
	 @(negedge CLK2X);
         #1 ROM_DOA[15:0] = data[15:0];
         @(posedge CLK2X);
      end
   endtask // rom_out

endmodule // test
